@@ -17,6 +17,11 @@ defaults = {
     # Only functions when always_rename is True
     'skip_file_on_error': True,
 
+    # Forcefully overwrite existing files when renaming or
+    # moving. This potentially destroys the old file. Default is False
+    'overwrite_destination_on_rename': False,
+    'overwrite_destination_on_move': False,
+
     # Verbose mode (debugging info)
     'verbose': False,
 
@@ -72,8 +77,9 @@ defaults = {
     # Move renamed files to directory?
     'move_files_enable': False,
 
-    # Seperation confirmation of moving or copying renamed file?
-    # If False, will move files when renaming.
+    # Separate confirmation of moving or copying renamed file?  If
+    # False, will move files when renaming. In batch mode, will never
+    # prompt.
     'move_files_confirmation': True,
 
     # If true, convert the variable/dynamic parts of the destination
@@ -83,6 +89,10 @@ defaults = {
     # then only the series name will be converted to lower case.
     'move_files_lowercase_destination': False,
 
+    # If True, the destination path includes the destination filename,
+    # for example: '/example/tv/%(seriesname)s/season %(seasonnumber)d/%(originalfilename)'
+    'move_files_destination_is_filepath': False,
+
     # Destination to move files to. Trailing slash is not necessary.
     # Use forward slashes, even on Windows. Realtive paths are realtive to
     # the existing file's path (not current working dir). A value of '.' will
@@ -95,20 +105,48 @@ defaults = {
     #                       variable episode_single and joined with episode_separator)
     'move_files_destination': '.',
 
+    # Same as above, only for date-numbered episodes. The following
+    # variables are available:
+    # - %(seriesname)s
+    # - %(year)s
+    # - %(month)s
+    # - %(day)s
+    'move_files_destination_date': '.',
+
+    # Force the move-files feature to always move the file.
+    #
+    # If False, when a file is moved between partitions (or from a
+    # network volume), the original is left untouched (i.e it is
+    # copied).  If True, this will delete the file from the original
+    # volume, after the copy has complete.
+    'always_move': False,
+    
+    # Allow user to copy files to specified move location without renaming files.
+    'move_files_only': False,
+
     # Patterns to parse input filenames with
     'filename_patterns': [
-        # [group] Show - 01-02 [Etc]
-        '''^\[.+?\][ ]? # group name
+        # [group] Show - 01-02 [crc]
+        '''^\[(?P<group>.+?)\][ ]?               # group name, captured for [#100]
         (?P<seriesname>.*?)[ ]?[-_][ ]?          # show name, padding, spaces?
         (?P<episodenumberstart>\d+)              # first episode number
         ([-_]\d+)*                               # optional repeating episodes
         [-_](?P<episodenumberend>\d+)            # last episode number
+        (?=                                      # Optional group for crc value (non-capturing)
+          .*                                     # padding
+          \[(?P<crc>.+?)\]                       # CRC value
+        )?                                       # End optional crc group
         [^\/]*$''',
 
-        # [group] Show - 01 [Etc]
-        '''^\[.+?\][ ]? # group name
-        (?P<seriesname>.*) # show name
-        [ ]?[-_][ ]?(?P<episodenumber>\d+)
+        # [group] Show - 01 [crc]
+        '''^\[(?P<group>.+?)\][ ]?               # group name, captured for [#100]
+        (?P<seriesname>.*)                       # show name
+        [ ]?[-_][ ]?                             # padding and seperator
+        (?P<episodenumber>\d+)                   # episode number
+        (?=                                      # Optional group for crc value (non-capturing)
+          .*                                     # padding
+          \[(?P<crc>.+?)\]                       # CRC value
+        )?                                       # End optional crc group
         [^\/]*$''',
 
         # foo s01e23 s01e24 s01e25 *
@@ -223,10 +261,10 @@ defaults = {
         \]?                                      # ] optional
         [^\\/]*$''',
 
-        # foo.s01.e01, foo.s01_e01
+        # foo.s01.e01, foo.s01_e01, "foo.s01 - e01"
         '''^((?P<seriesname>.+?)[ \._\-])?
         \[?
-        [Ss](?P<seasonnumber>[0-9]+)[\.\- ]?
+        [Ss](?P<seasonnumber>[0-9]+)[ ]?[\._\- ]?[ ]?
         [Ee]?(?P<episodenumber>[0-9]+)
         \]?
         [^\\/]*$''',
@@ -272,7 +310,7 @@ defaults = {
         .*$                                      # rest of file
         ''',
 
-        # show.name.e123.abc
+        # show name 2 of 6 - blah
         '''^(?P<seriesname>.+?)                  # Show name
         [ \._\-]                                 # Padding
         (?P<episodenumber>[0-9]+)                # 2
@@ -282,6 +320,35 @@ defaults = {
         ([\._ -]|$|[^\\/]*$)                     # More padding, then anything
         ''',
 
+        # Show.Name.Part.1.and.Part.2
+        '''^(?i)
+        (?P<seriesname>.+?)                        # Show name
+        [ \._\-]                                   # Padding
+        (?:part|pt)?[\._ -]
+        (?P<episodenumberstart>[0-9]+)             # Part 1
+        (?:
+          [ \._-](?:and|&|to)                        # and
+          [ \._-](?:part|pt)?                        # Part 2
+          [ \._-](?:[0-9]+))*                        # (middle group, optional, repeating)
+        [ \._-](?:and|&|to)                        # and
+        [ \._-]?(?:part|pt)?                       # Part 3
+        [ \._-](?P<episodenumberend>[0-9]+)        # last episode number, save it
+        [\._ -][^\\/]*$                            # More padding, then anything
+        ''',
+
+        # Show.Name.Part1
+        '''^(?P<seriesname>.+?)                  # Show name\n
+        [ \\._\\-]                               # Padding\n
+        [Pp]art[ ](?P<episodenumber>[0-9]+)      # Part 1\n
+        [\\._ -][^\\/]*$                         # More padding, then anything\n
+        ''',
+
+        # show name Season 01 Episode 20
+        '''^(?P<seriesname>.+?)[ ]?               # Show name
+        [Ss]eason[ ]?(?P<seasonnumber>[0-9]+)[ ]? # Season 1
+        [Ee]pisode[ ]?(?P<episodenumber>[0-9]+)   # Episode 20
+        [^\\/]*$''',                              # Anything
+
         # foo.103*
         '''^(?P<seriesname>.+)[ \._\-]
         (?P<seasonnumber>[0-9]{1})
@@ -305,20 +372,41 @@ defaults = {
     # Formats for renamed files. Variations for with/without episode,
     # and with/without season number.
     'filename_with_episode':
-     '%(seriesname)s - [%(seasonno)02dx%(episode)s] - %(episodename)s%(ext)s',
+     '%(seriesname)s - [%(seasonnumber)02dx%(episode)s] - %(episodename)s%(ext)s',
     'filename_without_episode':
-     '%(seriesname)s - [%(seasonno)02dx%(episode)s]%(ext)s',
+     '%(seriesname)s - [%(seasonnumber)02dx%(episode)s]%(ext)s',
+
+    # Seasonless filenames.
     'filename_with_episode_no_season':
       '%(seriesname)s - [%(episode)s] - %(episodename)s%(ext)s',
     'filename_without_episode_no_season':
      '%(seriesname)s - [%(episode)s]%(ext)s',
 
+    # Date based filenames.
+    # Series - [2012-01-24] - Ep name.ext
     'filename_with_date_and_episode':
      '%(seriesname)s - [%(episode)s] - %(episodename)s%(ext)s',
-
     'filename_with_date_without_episode':
      '%(seriesname)s - [%(episode)s]%(ext)s',
 
+    # Anime filenames.
+    # [AGroup] Series - 02 - Some Ep Name [CRC1234].ext
+    # [AGroup] Series - 02 [CRC1234].ext
+    'filename_anime_with_episode':
+     '[%(group)s] %(seriesname)s - %(episode)s - %(episodename)s [%(crc)s]%(ext)s',
+
+    'filename_anime_without_episode':
+     '[%(group)s] %(seriesname)s - %(episode)s [%(crc)s]%(ext)s',
+
+    # Same, without CRC value
+    'filename_anime_with_episode_without_crc':
+     '[%(group)s] %(seriesname)s - %(episode)s - %(episodename)s%(ext)s',
+
+    'filename_anime_without_episode_without_crc':
+     '[%(group)s] %(seriesname)s - %(episode)s%(ext)s',
+
+
+
     # Used to join multiple episode names together
     'multiep_join_name_with': ', ',
 
@@ -334,4 +422,14 @@ defaults = {
     
     # Forced Name to use
     #'forced_name': None,
+    
+    # replace series names before/after passing to TVDB
+    # input replacements are regular expressions for the series as parsed from
+    # filenames, for instance adding or removing the year, or expanding abbreviations
+    'input_series_replacements': {},
+    
+    # output replacements are for transforms of the TVDB series names
+    # since these are perfectly predictable, they are simple strings
+    # not regular expressions
+    'output_series_replacements': {},
 }
